require_dependency "binect_importer/application_controller"
require 'dropbox_sdk'
require 'securerandom'

# This configuration will be added by the Rails-Application
import_config = Rails.application.config.import_config[:dropbox]
APP_KEY       = "jdt9ez43gc7zna1" #import_config[:APP_KEY]
APP_SECRET    = "g1veqjyvoflxyyy" #import_config[:APP_SECRET]
ROOT_PATH     = import_config[:ROOT_PATH]
IMPORTER_TYPE = "dropbox"

module BinectImporter
    class DropboxRegistrationsController < AuthenticatedController
        def start_registration
            authorize_url = get_web_auth().start()
            Rails.logger.info "Start Dropbox registration for user: #{current_user}. Now redirect to URL: #{authorize_url}"

            redirect_to authorize_url
        end

        # GET /import_registration/dropbox_finish
        def finish_registration
            begin
                access_token, user_id, url_state = get_web_auth.finish(params)

                Rails.logger.info "Registration succeeded for user: #{current_user}."

                # Check whethere this user is already authorized
                importer_data_index = current_user.authenticated_importers.index { |importer_hash| importer_hash[:type] = IMPORTER_TYPE }
                importer_data = { class_name: "BinectImporter::DropboxImporter", type: IMPORTER_TYPE, access_token: access_token, user_id: user_id, url_state: url_state }
                if importer_data_index.nil?
                    # Add new importer_data
                    Rails.logger.info "Saving new access token"
                    current_user.authenticated_importers.push(importer_data)
                else
                    # Replace importer data
                    Rails.logger.info "Updating new access token"
                    current_user.authenticated_importers[importer_data_index] = importer_data
                end
                current_user.save

                redirect_to ROOT_PATH
            rescue DropboxOAuth2Flow::BadRequestError => e
                render :text => "Error in OAuth 2 flow: Bad request: #{e}"
            rescue DropboxOAuth2Flow::BadStateError => e
                logger.info("Error in OAuth 2 flow: No CSRF token in session: #{e}")
                redirect_to(:action => 'start_registration')
            rescue DropboxOAuth2Flow::CsrfError => e
                logger.info("Error in OAuth 2 flow: CSRF mismatch: #{e}")
                render :text => "CSRF error"
            rescue DropboxOAuth2Flow::NotApprovedError => e
                render :text => "Not approved?  Why not, bro?"
            rescue DropboxOAuth2Flow::ProviderError => e
                logger.info "Error in OAuth 2 flow: Error redirect from Dropbox: #{e}"
                render :text => "Strange error."
            rescue DropboxError => e
                logger.info "Error getting OAuth 2 access token: #{e}"
                render :text => "Error communicating with Dropbox servers."
            end
        end

        def unlink
            get_dropbox_client.disable_access_token
        end

        private
        def get_web_auth
            return DropboxOAuth2Flow.new(APP_KEY, APP_SECRET, url_for(:action => 'finish_registration'),
                                         session, :dropbox_auth_csrf_token, "de")
        end

        def get_dropbox_client
            dropbox_data = current_user.authenticated_importers.index { |importer_hash| importer_hash[:type] = IMPORTER_TYPE }
            fail "Not registered" if dropbox_data.nil? || dropbox_data[:access_token]

            access_token = dropbox_data[:access_token]
            DropboxClient.new(access_token)
        end

    end
end
